package com.gic.haoban.manage.service.service.out.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import cn.hutool.core.collection.CollectionUtil;

import com.gic.haoban.base.api.common.Constant;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.gic.api.base.commons.JSONResponse;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.EntityUtil;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.contacts.manage.api.dto.SyncJsonColumnDTO;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.service.dao.mapper.DepartmentMapper;
import com.gic.haoban.manage.service.entity.TabHaobanDepartment;
import com.gic.haoban.manage.service.entity.TabHaobanStaffDepartmentRelated;
import com.gic.haoban.manage.service.entity.TabHaobanWxApplication;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterpriseRelated;
import com.gic.haoban.manage.service.service.DepartmentService;
import com.gic.haoban.manage.service.service.StaffDepartmentRelatedService;
import com.gic.haoban.manage.service.service.WxApplicationService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.wechat.api.service.qywx.QywxDepartmentApiService;
import com.github.pagehelper.PageHelper;

@Service
public class DepartmentApiServiceImpl implements DepartmentApiService {
	
	private static Logger logger  = LoggerFactory.getLogger(DepartmentApiServiceImpl.class);
	
	@Autowired
	private DepartmentService departmentService;
	@Autowired
	private StaffDepartmentRelatedService staffDepartmentService;
	
	@Autowired
	private DepartmentMapper departmentMapper;
	
	@Autowired
	private com.gic.enterprise.api.service.DepartmentService gicDepartmentService;
	
	@Autowired
	private QywxDepartmentApiService qywxDepartmentApiService;
	
	@Autowired
	private WxEnterpriseService wxEnterpriseService;
	
	@Autowired
	private WxApplicationService wxApplicationService;
	
	
	@Override
	public List<DepartmentDTO> listByParentId(String parentId) {
		List<TabHaobanDepartment> list = departmentService.listByParentId(parentId);
		
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}

	@Override
	public DepartmentDTO selectById(String departmentId) {
		TabHaobanDepartment tab = departmentService.selectById(departmentId);
		return EntityUtil.changeEntityByJSON(DepartmentDTO.class, tab);
	}

	@Override
	public HaobanResponse add(DepartmentDTO department) {
		HaobanResponse hr = new HaobanResponse();
		hr.setErrorCode(1);
		//调微信的新增接口
		String wxDepartmentId = "";
		WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(department.getWxEnterpriseId());
		TabHaobanWxApplication application = wxApplicationService.selectByWxEnterpriseIdAndApplicationType(department.getWxEnterpriseId(),1);
		logger.info("【部门新增】enterpriseDTO = {}",JSON.toJSONString(enterpriseDTO));
		logger.info("【部门新增】application = {}",JSON.toJSONString(application));
		if(enterpriseDTO != null) {
			if(application != null) {
				com.gic.wechat.api.dto.qywx.DepartmentDTO dto = new com.gic.wechat.api.dto.qywx.DepartmentDTO();
				dto.setCorpid(enterpriseDTO.getCorpid());
				dto.setName(department.getDepartmentName());
				dto.setOrder(department.getSort());
				TabHaobanDepartment parent = departmentService.selectById(department.getParentDepartmentId());
				dto.setParentid(Integer.valueOf(parent.getWxDepartmentId()));
				logger.info("【部门新增】corpid() = {},siteId={},dto={}",enterpriseDTO.getCorpid(),application.getSiteId(),JSON.toJSONString(dto));
				JSONResponse jp = qywxDepartmentApiService.createDepartment(enterpriseDTO.getCorpid(), application.getSiteId(), dto);
				logger.info("【部门新增】jp = {}",JSON.toJSONString(jp));
				if(jp.getErrorCode() != 0) {
					hr.setErrorCode(0);
					hr.setMessage("微信新增部门失败");
					return hr;
				}
				wxDepartmentId = jp.getResult().toString();
				logger.info("【部门新增】wxDepartmentId = {}",wxDepartmentId);
				department.setWxDepartmentId(wxDepartmentId);
				department.setLevel(parent.getLevel() + 1);
				String departmentId = departmentService.add(department);
				department.setDepartmentId(departmentId);
				hr.setResult(department);
			}
		}
		
		return hr;
	}

	@Override
	public HaobanResponse edit(DepartmentDTO department) {
		HaobanResponse hr = new HaobanResponse();
		hr.setErrorCode(1);
		//调微信的修改接口
		TabHaobanDepartment tab = departmentService.selectById(department.getDepartmentId());
		logger.info("【部门修改】tab={}",JSON.toJSONString(tab));
		WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(tab.getWxEnterpriseId());
		TabHaobanWxApplication application = wxApplicationService.selectByWxEnterpriseIdAndApplicationType(tab.getWxEnterpriseId(),1);
		if(enterpriseDTO != null) {
			if(application != null) {
				com.gic.wechat.api.dto.qywx.DepartmentDTO dto = new com.gic.wechat.api.dto.qywx.DepartmentDTO();
				dto.setCorpid(enterpriseDTO.getCorpid());
				dto.setName(department.getDepartmentName());
				dto.setOrder(department.getSort());
				TabHaobanDepartment parent = departmentService.selectById(department.getParentDepartmentId());
				dto.setParentid(Integer.valueOf(parent.getWxDepartmentId()));
				dto.setId(Integer.valueOf(tab.getWxDepartmentId()));
				JSONResponse jp = qywxDepartmentApiService.updateDepartment(enterpriseDTO.getCorpid(), application.getSiteId(), dto);
				logger.info("【部门修改】jp={}",JSON.toJSONString(jp));
				if(jp.getErrorCode() != 0) {
					hr.setErrorCode(0);
					hr.setMessage("微信修改部门失败");
					return hr;
				}
			}
		}
		
		departmentService.edit(department);
		return hr;
	}

	@Override
	public HaobanResponse del(String departmentId) {
		HaobanResponse hr = new HaobanResponse();
		hr.setErrorCode(1);
		
		List<TabHaobanDepartment> list = departmentService.listByParentId(departmentId);
		if(!list.isEmpty()) {
			hr.setErrorCode(0);
			hr.setMessage("存在子部门，删除部门失败");
			return hr;
		}
		
		List<TabHaobanStaffDepartmentRelated> staffList = staffDepartmentService.listByDepartmentId(departmentId);
		if(!staffList.isEmpty()) {
			hr.setErrorCode(0);
			hr.setMessage("存在成员，删除部门失败");
			return hr;
		}
		//调微信的新增接口
		TabHaobanDepartment department = departmentService.selectById(departmentId);
		WxEnterpriseDTO enterpriseDTO = wxEnterpriseService.selectById(department.getWxEnterpriseId());
		TabHaobanWxApplication application = wxApplicationService.selectByWxEnterpriseIdAndApplicationType(department.getWxEnterpriseId(),1);
		if(enterpriseDTO != null) {
			if(application != null) {
				JSONResponse jp = qywxDepartmentApiService.deleteDepartment(enterpriseDTO.getCorpid(), application.getSiteId(), Integer.parseInt(department.getWxDepartmentId()));
				if(jp.getErrorCode() != 0) {
					hr.setErrorCode(0);
					hr.setMessage("微信删除部门失败");
					return hr;
				}
			}
		}
		
		departmentService.del(departmentId);
		return hr;
	}

	@Override
	public DepartmentDTO selectByRelatedId(String relatedId) {
		TabHaobanDepartment tab = departmentService.selectByRelatedId(relatedId);
		return EntityUtil.changeEntityByJSON(DepartmentDTO.class, tab);
	}

	@Override
	public HaobanResponse recycle(String departmentId) {
		HaobanResponse hr = new HaobanResponse();
		hr.setErrorCode(1);
		departmentService.recycle(departmentId);
		return hr;
	}

	@Override
	public Page<DepartmentDTO> pageDepartmentByParams(BasePageInfo pageInfo, String wxEnterpriseId, String keyword,
			Integer storeFlag, Integer recycleFlag) {
		PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
		return  PageUtil.changePageHelperToCurrentPage(departmentService.pageDepartmentByParams(wxEnterpriseId,keyword,storeFlag,recycleFlag),DepartmentDTO.class);
	}

	@Override
	public HaobanResponse repairRecycle(String departmentId) {
		HaobanResponse hr = new HaobanResponse();
		hr.setErrorCode(0);
		departmentService.repairRecycle(departmentId);
		return hr;
		
	}

	@Override
	public List<DepartmentDTO> listByDepartmentIds(List<String> departmentIds,
			int storeFlag) {
		List<TabHaobanDepartment> list= departmentMapper.listByDepartmentIds(departmentIds, storeFlag);
		if(list==null){
			return new ArrayList<DepartmentDTO>();
		}
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}

	@Override
	public DepartmentDTO getRootByEnterpriseId(String wxEnterpriseId) {
		
		return departmentService.getRootByEnterpriseId(wxEnterpriseId);
	}

	@Override
	public List<DepartmentDTO> listStoreByWxEnterpriseId(String wxEnterpriseId) {
		List<TabHaobanDepartment> list = departmentMapper.listStoreByWxEnterpriseId(wxEnterpriseId);
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}

	@Override
	public Page<DepartmentDTO> pageFullStoreByWxEnterpriseId(
			String wxEnterpriseId,String search,List<String> storeIds, int count,BasePageInfo pageInfo) {
		PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
		if(CollectionUtil.isNotEmpty(storeIds)&&StringUtils.isNotEmpty(search)){
			return PageUtil.changePageHelperToCurrentPage( departmentMapper.pageFullStoreByWxEnterpriseId2(wxEnterpriseId,search, storeIds,count),DepartmentDTO.class);
		}
		return PageUtil.changePageHelperToCurrentPage( departmentMapper.pageFullStoreByWxEnterpriseId(wxEnterpriseId,search, storeIds,count),DepartmentDTO.class);
	}

	@Override
	public void syncGicStore(String fieldListString) {
		logger.info("【同步门店】fieldList={}",fieldListString);
		List<SyncJsonColumnDTO> fieldList = JSONArray.parseArray(fieldListString,SyncJsonColumnDTO.class);
		if(fieldList.isEmpty()){
	        return;
	    }
		
		Integer status = 0;
		String storeName = "";
		String departmentId = "";
		String storeId = "";
		for (SyncJsonColumnDTO syncJsonColumnDTO:fieldList){
	    	String name = StringUtil.camelName(syncJsonColumnDTO.getName());
	        String value = syncJsonColumnDTO.getValue();
	        if("status".equals(name)) {
	        	status = Integer.parseInt(value);
	        }
	        if("storeName".equals(name)) {
	        	storeName = value;
	        }
	        if("departmentId".equals(name)) {
	        	departmentId = value;
	        }
	        if("storeId".equals(name)) {
	        	storeId = value;
	        }
		}
		DepartmentDTO department = departmentService.getByRelatedId(storeId);
		if(department == null) {
			if(status != null && status == 2) {
				DepartmentDTO parent = getParent(departmentId);
				if(parent == null) {
					TabHaobanWxEnterpriseRelated tab = null;
					//parent = getRootByEnterpriseId(wxEnterpriseId)；
				}
			}
		}
		
	        
		
	}
	/**
	 * 向上找父节点
	 * @param departmentId
	 * @return
	 */
	private DepartmentDTO getParent(String departmentId) {
		DepartmentDTO department = departmentService.getByRelatedId(departmentId);
		if(department == null) {
			com.gic.enterprise.api.dto.DepartmentDTO dto = gicDepartmentService.getDeptment(departmentId);
			String parentId = dto.getParentDepartId();
			getParent(parentId);
		}
		return department;
	}

	@Override
	public void syncGicDepartment(String fieldListString) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public Page<DepartmentDTO> pageByParentId(String parentId,String keyword, BasePageInfo pageInfo) {
		PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
		return  PageUtil.changePageHelperToCurrentPage(departmentService.pageByParentId(parentId,keyword),DepartmentDTO.class);
	}

	@Override
	public List<DepartmentDTO> listByChainId(String chainId,String wxEnterpriseId) {
		List<TabHaobanDepartment> list = departmentService.listByChainId(chainId,wxEnterpriseId);
		return EntityUtil.changeEntityListByJSON(DepartmentDTO.class, list);
	}

	@Override
	public void initwxDepartment(String corpid, String suiteid, String wxEnterpriseId) {
		List<com.gic.wechat.api.dto.qywx.DepartmentDTO> list = this.qywxDepartmentApiService.listDepartment(corpid, suiteid, null);
		logger.info("企业所有部门数据：{}", JSON.toJSONString(list));
		if(CollectionUtils.isNotEmpty(list)){
			this.addAllDepartment(list, 0, wxEnterpriseId);
		}
	}


	private void addAllDepartment(List<com.gic.wechat.api.dto.qywx.DepartmentDTO> list, Integer parentId, String wxEnterpriseId){
		int level = 1;
		String chainId = "0";
		String chainName = "0";
		String pId = "0";
		if(parentId != 0){
			TabHaobanDepartment parent = this.departmentService.getByWxId(parentId + "", wxEnterpriseId);
			if(parent != null){
				level = parent.getLevel() + 1;
				chainId = parent.getChainId() + Constant.ID_SEPARATOR + parent.getDepartmentId();
				chainName = parent.getChainName() + Constant.NAME_SEPARATOR + parent.getChainName();
				pId = parent.getDepartmentId();
			}
		}
		for(com.gic.wechat.api.dto.qywx.DepartmentDTO departmentDTO : list){
			if(parentId == departmentDTO.getParentid()){
				TabHaobanDepartment exist = this.departmentService.getByWxId(departmentDTO.getId() + "", wxEnterpriseId);
				logger.info("分组是否存在：{}", JSON.toJSONString(exist));
				if(exist == null){
					com.gic.haoban.manage.api.dto.DepartmentDTO dto = new com.gic.haoban.manage.api.dto.DepartmentDTO();
					dto.setDepartmentName(departmentDTO.getName());
					dto.setLevel(level);
					dto.setWxEnterpriseId(wxEnterpriseId);
					dto.setWxDepartmentId(departmentDTO.getId()+"");
					dto.setParentDepartmentId(pId);
					dto.setChainId(chainId);
					dto.setChainName(chainName);
					dto.setIsStore(0);
					this.departmentService.add(dto);
				}
				this.addAllDepartment(list, departmentDTO.getId(), wxEnterpriseId);
			}
		}
	}

	@Override
	public Integer departmentRecycleCount(String wxEnterpriseId) {
		return departmentService.departmentRecycleCount(wxEnterpriseId);
	}
}
